@@ -1,6 +1,6 @@
 use crate::base::*;
 use alloc::boxed::Box;
-use components_arena::{Component, Id, Arena, NewtypeComponentId};
+use components_arena::{Component, ComponentId, Id, Arena, NewtypeComponentId, RawId};
 use core::any::{Any, TypeId};
 use core::fmt::Debug;
 use downcast_rs::{Downcast, impl_downcast};
@@ -163,7 +163,7 @@ impl Drop for Bindings {
     }
 }
 
-trait AnyBindingNodeSources: Debug + Downcast {
+trait AnyBindingNodeSources: Downcast {
     type Value: Convenient;
     fn unhandle(&mut self, state: &mut dyn State);
     fn get_value(&self) -> Option<Self::Value>;
@@ -175,6 +175,7 @@ impl_downcast!(AnyBindingNodeSources assoc Value where Value: Convenient);
 #[derive(Educe)]
 #[educe(Debug)]
 struct BindingNode<T: Convenient> {
+    #[educe(Debug(ignore))]
     sources: Box<dyn AnyBindingNodeSources<Value=T>>,
     target: Option<Box<dyn Target<T>>>,
 }
@@ -324,7 +325,7 @@ macro_rules! binding_n {
                 #[educe(Debug(ignore))]
                 dispatch: fn(
                     &mut dyn State,
-                    Glob < AnyBindingBase, P >,
+                    Glob <  P >,
                     $( < < [< S $i >] as Source > ::Cache as SourceCache< [< S $i >] ::Value > >::Value ),*
                 ) -> BYield<T>,
             }
@@ -378,7 +379,7 @@ macro_rules! binding_n {
                     param: P,
                     dispatch: fn(
                         &mut dyn State,
-                        Glob < AnyBindingBase, P >,
+                        Glob < P >,
                         $( < < [< S $i >] as Source > ::Cache as SourceCache< [< S $i >] ::Value > >::Value ),*
                     ) -> BYield<T>,
                 ) -> Self {
@@ -400,22 +401,22 @@ macro_rules! binding_n {
                     [< BindingExt $n >] (id, PhantomType::new())
                 }
 
-                fn param_ref(arena: &dyn Any, id: AnyBindingBase) -> &P {
+                fn param_ref(arena: &dyn Any, id: RawId) -> &P {
                     let bindings = arena.downcast_ref::<Bindings>().unwrap();
-                    let node = bindings.0[id.0].0.downcast_ref::<BindingNode<T>>().unwrap();
+                    let node = bindings.0[Id::from_raw(id)].0.downcast_ref::<BindingNode<T>>().unwrap();
                     let sources = node.sources.downcast_ref::< [< BindingExt $n NodeSources >] <P, $( [< S $i >] ,)* T>>().unwrap();
                     &sources.param
                 }
 
-                fn param_mut(arena: &mut dyn Any, id: AnyBindingBase) -> &mut P {
+                fn param_mut(arena: &mut dyn Any, id: RawId) -> &mut P {
                     let bindings = arena.downcast_mut::<Bindings>().unwrap();
-                    let node = bindings.0[id.0].0.downcast_mut::<BindingNode<T>>().unwrap();
+                    let node = bindings.0[Id::from_raw(id)].0.downcast_mut::<BindingNode<T>>().unwrap();
                     let sources = node.sources.downcast_mut::< [< BindingExt $n NodeSources >] <P, $( [< S $i >] ,)* T>>().unwrap();
                     &mut sources.param
                 }
 
                 #[allow(dead_code)]
-                fn param_descriptor() -> GlobDescriptor<AnyBindingBase, P> {
+                fn param_descriptor() -> GlobDescriptor<P> {
                     GlobDescriptor {
                         arena: TypeId::of::<Bindings>(),
                         field_ref: Self::param_ref,
@@ -549,7 +550,7 @@ macro_rules! binding_n {
 
                         let target = node.target.clone();
                         let param = Glob {
-                            id: AnyBindingBase(self.binding),
+                            id: self.binding.into_raw(),
                             descriptor: < [< BindingExt $n >] <P, $( [< S $j >] ,)* T> > ::param_descriptor
                         };
                         if let BYield(Some(value)) = (sources.dispatch)(state, param, $( [< value_ $j >] ),*) {
@@ -560,8 +561,8 @@ macro_rules! binding_n {
             )*
 
             #[derive(Educe)]
-            #[educe(Debug)]
-            struct [< Binding $n NodeSources >] <P: Convenient, $( [< S $i >] : Source, )* T: Convenient> {
+            #[educe(Debug(bound="P: Debug"))]
+            struct [< Binding $n NodeSources >] <P, $( [< S $i >] : Source, )* T: Convenient> {
                 param: P,
                 $(
                     [< source_ $i >] : Option<(Box<dyn HandlerId>, [< S $i >] ::Cache )>,
@@ -571,7 +572,7 @@ macro_rules! binding_n {
             }
 
             impl<
-                P: Convenient,
+                P: Clone + 'static,
                 $( [< S $i >] : Source + 'static, )*
                 T: Convenient
             > AnyBindingNodeSources for [< Binding $n NodeSources >] <P, $( [< S $i >] , )* T> {
@@ -622,7 +623,7 @@ macro_rules! binding_n {
             }
 
             impl<
-                P: Convenient,
+                P: Clone + 'static,
                 $( [< S $i >] : Source + 'static, )*
                 T: Convenient
             > [< Binding $n >] <P, $( [< S $i >] , )* T> {
@@ -743,7 +744,7 @@ macro_rules! binding_n {
                 }
 
                 impl<
-                    P: Convenient,
+                    P: Clone + 'static,
                     $( [< S $j >] : Source + 'static, )*
                     T: Convenient
                 > AnyHandler for [< Binding $n Source $i Handler >] <P, $( [< S $j >] , )* T >  {
@@ -756,7 +757,7 @@ macro_rules! binding_n {
                 }
 
                 impl<
-                    P: Convenient,
+                    P: Clone + 'static,
                     $( [< S $j >] : Source + 'static, )*
                     T: Convenient
                 > Handler< [< S $i >] ::Value > for [< Binding $n Source $i Handler >] <P, $( [< S $j >] , )* T >  {
